#ifndef AUDIO_H
#define AUDIO_H
#include <windows.h>
#include <mmsystem.h>

/*
 * buffer is 160 kilobytes in this configuration
 * which is substantially less than winamp's 320.
 * (gives us about 850 ms latency). You can go much
 * smaller but with the risk of buffer underrun.
 */
#define BLOCK_COUNT 20
#define BLOCK_SIZE  8192

#define BLOCK_COUNT_MAX 50
#define BLOCK_COUNT_MIN 2
#define BLOCK_SIZE_MAX 65536
#define BLOCK_SIZE_MIN 1024

/*
 * configuration structure
 */
typedef struct {
        int block_count;
        int block_size;
        int volume_enabled;
        int reverse_balance;
        int device;
        int pcmconv_enabled;
} config_t;

typedef struct {
        int samplerate;
        int bitspersamp;
        int numchannels;
        int conv;
} format_t;

typedef struct {
        int active;
        int total_buffer_size;
        int total_buffer_usage;
        int block_size;
        int block_usage;
        int block_count;
        int bytes_written;
        int bytes_played;
        int start_position;
        int latency;
        format_t format;
} stats_t;

void OutputClose();
void OutputQuit();
int OutputOpen(int samplerate, int numchannels, int bitspersamp, int bufferlenms, int prebufferms);
int OutputPause(int pause);
int OutputPlay(char *buffer, int size);
void OutputWrite(struct buffer *buffer);
void OutputDrain(int time);
void OutputSetVolume(int volume);
void OutputConfig(HWND hwndParent);
void OutputAbout(HWND hwndParent);
void OutputInit();
int CanWrite();
int IsPlaying();
void OutputSetPan(int pan);
int GetOutputTime(); 
int GetWrittenTime();
static void CALLBACK waveout_callback(HWAVEOUT device, UINT msg, DWORD inst, DWORD parm1, DWORD parm2);

void get_statistics(stats_t *stats);
void get_config(config_t *config);
void set_config(const config_t *config);
int config_load(config_t *config);
void config_save(const config_t *config);
void config_set_defaults(config_t *config);
BOOL CALLBACK config_dialog_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);


#endif