#include "../stdafx.h"
#include "WaveBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int outputTime, writtenTime, waveCount;
int pause = 0;
int currentPan = 0;
HANDLE ThreadWaitEvent = NULL;
CDuration dura, dura1;

CWaveBox::CWaveBox() {
    // init wave(s) counter
    wload = 0;
    wwrite = 0;
    outputTime = 0;
    writtenTime = 0;

    // thread suspended < used for resuming thread at first play >
    run = 0;

    // init cs
    InitializeCriticalSection(&cs);

    // create suspended player thread
    thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) PlayThread, (LPVOID)this, NULL, NULL);
    ThreadWaitEvent = CreateEvent(NULL, false, false, NULL);
    dura.Start();

    // alloc mem for interface(s)
    for(unsigned int i = 0; i < SUPPORT_INTERFACES; i++) {
        INTERF[i].wblock = allocateBlocks(BLOCK_SIZE, BLOCK_COUNT);
        INTERF[i].wfreeblock = BLOCK_COUNT;
        INTERF[i].wcurrblock = 0;
        INTERF[i].state = INT_FREE;
        INTERF[i].wpos = 0;
    }

    // init msg
    for(i = 0; i < SUPPORT_WAVES; i++) {
        W[i].WMSG = WMSG_WAIT;
        W[i].size = 0;
    }
}


WAVEHDR * CWaveBox::allocateBlocks(int size, int count) {
    unsigned char * buffer;
    int i;
    WAVEHDR *blocks;
    DWORD totalBufferSize = (size + sizeof(WAVEHDR)) * count;

    //  allocate memory for the entire set in one go
    if((buffer = (UCHAR *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, totalBufferSize)) == NULL)
        return NULL;

    // and set up the pointers to each bit
    blocks = (WAVEHDR *) buffer;
    buffer += sizeof(WAVEHDR) * count;
    for(i = 0; i < count; i++) {
        blocks[i].dwBufferLength = size;
        blocks[i].lpData = (CHAR *) buffer;
        buffer += size;
    }

    return blocks;
}

void CWaveBox::SetFormat(WAVEFORMATEX *format, unsigned int channels, unsigned int nSamplerate,
                         unsigned int nBitspersamp) {
    unsigned int block_al;
    unsigned long bytes_ps;

    block_al = channels * nBitspersamp / 8;
    bytes_ps = nSamplerate * block_al;

    format->wFormatTag = WAVE_FORMAT_PCM;
    format->nChannels = channels;
    format->nSamplesPerSec = nSamplerate;
    format->nAvgBytesPerSec = bytes_ps;
    format->nBlockAlign = block_al;
    format->wBitsPerSample = nBitspersamp;
    format->cbSize = 0;
}

CWaveBox::~CWaveBox() {
    unsigned long exit = 0;

    if(run) // thread resumed
    {
        //        Sleep(5000);
        // set thread close message
        EnterCriticalSection(&cs);
        TMSG = TMSG_CLOSE;
        LeaveCriticalSection(&cs);

        do // wait for soft close
        {
            GetExitCodeThread(thread, &exit); 
            Sleep(1);
        }
        while(exit != THREAD_EXIT);
    }
    else // thread suspended    
    {
        // hard close 
        GetExitCodeThread(thread, &exit); 
        TerminateThread(thread, exit);
    }


    // release wave(s)
    for(unsigned int i = 0; i < wload; i++)
        free(W[i].data);

    // release interface(s)
    for(i = 0; i < SUPPORT_INTERFACES; i++)
        freeBlocks(INTERF[i].wblock); 

    // del cs
    DeleteCriticalSection(&cs);
}

void CWaveBox::freeBlocks(WAVEHDR *blockArray) {
    // and this is why allocateBlocks works the way it does
    HeapFree(GetProcessHeap(), 0, blockArray);
}


int CWaveBox::Load(int nSamplerate, int nNumchannels, int nBitspersamp, int bufferlenms, int prebufferms) {
    int i;
    wwrite = 0;
    wload = 0;
    waveCount = 0;

    pause = 0;

    for(i = 0; i < SUPPORT_INTERFACES; i++) {
        INTERF[i].wfreeblock = BLOCK_COUNT;
        INTERF[i].wcurrblock = 0;
        INTERF[i].state = INT_FREE;
        INTERF[i].wpos = 0;
    }

    // fill WAVEFORMATEX from wave header
    for(i = 0; i < SUPPORT_WAVES; i++) {
        SetFormat(&W[i].wfx, nNumchannels, nSamplerate, nBitspersamp);
        if((W[i].data = (char *) calloc(BLOCK_SIZE, BLOCK_COUNT)) == NULL)
            return -1;
        W[i].size = 0;
        W[i].WMSG = WMSG_WAIT;
        W[i].awrite = &wwrite;
    }
    return 0;
}


int CWaveBox::Play(char *buf, int len) {
    /* prepare block */
    if(W[wload].size + len >= BLOCK_SIZE * BLOCK_COUNT) {
        int first = BLOCK_SIZE *BLOCK_COUNT - W[wload].size;
        memcpy(W[wload].data + W[wload].size, buf, first); 
        W[wload].size += first;
        W[wload].sizeLeft += first;


        // set play message
        EnterCriticalSection(&cs);
        W[wload].WMSG = WMSG_START;
        wload++;
        waveCount++;
        wload %= SUPPORT_WAVES;
        LeaveCriticalSection(&cs);

        while(waveCount == SUPPORT_WAVES) {
            Sleep(1);
            //TRACE(_T("ThreadWaitEvent - J'attends -\n"));
            //WaitForSingleObject(ThreadWaitEvent, INFINITE);
            //TRACE(_T("ThreadWaitEvent - J'ai fini -\n"));
        }
        dura.Stop();
        TRACE(_T("Play Dure %g"), dura.GetDuration() / 1000.0);
        dura.Start();

        if(W[wload].data == NULL && (W[wload].data = (char *) calloc(BLOCK_SIZE, BLOCK_COUNT)) == NULL)
            return -1;
        W[wload].size = 0;
        memcpy(W[wload].data + W[wload].size, buf + first, len - first); 
        W[wload].size += len - first;
        //Sleep(77);
    }
    else {
        memcpy(W[wload].data + W[wload].size, buf, len); 
        W[wload].size += len;
    }
    writtenTime += 1000 * len
                 / (W[wload].wfx.wBitsPerSample / 8 * W[wload].wfx.nChannels * W[wload].wfx.nSamplesPerSec);

    return 0;
}

void CWaveBox::OutputSetVolume(int volume) {
    DWORD maxLevel = 0xFFFF;
    DWORD minLevel = 0;

    DWORD leftLevel = (volume * (maxLevel - minLevel) / 256) * (128 + currentPan) / 256 << 16;
    DWORD rightLevel = (volume * (maxLevel - minLevel) / 256) * (128 - currentPan) / 256;
    DWORD volumeLevel = leftLevel + rightLevel;
    for(int i = 0; i < SUPPORT_INTERFACES; i++)
        waveOutSetVolume(INTERF[i].dev, volumeLevel);
}

int CWaveBox::AddInterface(HWAVEOUT *dev, WAVEFORMATEX *wfx, volatile int *wfreeblock) {
    CDuration dur;
    dur.Start();
    // check for free device
    if(!waveOutGetNumDevs())
        return -1;

    // try to open the default wave device. WAVE_MAPPER is
    // a constant defined in mmsystem.h, it always points to the
    // default wave device on the system (some people have 2 or
    // more sound cards).

    if(waveOutOpen(dev, WAVE_MAPPER, wfx, (DWORD) waveOutProc, (DWORD) wfreeblock, CALLBACK_FUNCTION)
    != MMSYSERR_NOERROR)
        return -1;

    dur.Stop();
    //TRACE(_T("AddInterface Dure %g"),dur.GetDuration()/1000.0);

    return 1;
}


int CWaveBox::RemoveInterface(HWAVEOUT dev) {
    CDuration dur;
    dur.Start();
    // try to close given wave device / interface.

    if(waveOutClose(dev) != MMSYSERR_NOERROR)
        return -1;
    dur.Stop();
    //TRACE(_T("RemoveInterface Dure %g"),dur.GetDuration()/1000.0);

    return 1;
}

int CWaveBox::CanWrite() {
    int res = ((SUPPORT_WAVES - (SUPPORT_WAVES + wload - wwrite) % SUPPORT_WAVES)) * (BLOCK_COUNT *BLOCK_SIZE)
            - W[wload].size;
    if(pause)
        return 0;
    return res;
}
int CWaveBox::GetOutputTime() {
    if(pause == 1)
        return 0;
    return outputTime;
}

int CWaveBox::IsPlaying() {
    if(waveCount != 0)
        return 1;
    return 0;
}

void CWaveBox::OutputSetPan(int pan) {
    currentPan = pan;
}

int CWaveBox::GetWrittenTime() {
    return writtenTime;
}


int CWaveBox::OutputPause(int flag) {
    for(int i = 0; i < SUPPORT_INTERFACES; i++)
        if(flag && !pause)
            waveOutPause(INTERF[i].dev);
        else if(!flag && pause)
            waveOutRestart(INTERF[i].dev);
    pause = flag;
    return pause;
}

static void CALLBACK waveOutProc(HWAVEOUT   hWaveOut, UINT      uMsg, DWORD     dwInstance, DWORD       dwParam1,
                                 DWORD     dwParam2) {
    // pointer to free block counter
    int *freeBlockCounter = (int *) dwInstance;

    WAVEHDR *currentWHDR = (WAVEHDR *) dwParam1;

    // ignore calls that occur due to openining and closing the device.
    if(uMsg != WOM_DONE)
        return;
    WAVE *currentWave = (WAVE *) currentWHDR->dwUser;
    //TODO Corriger la taille
    outputTime += BLOCK_SIZE * 1000
                / (currentWave->wfx.wBitsPerSample / 8 * currentWave->wfx.nChannels * currentWave->wfx.nSamplesPerSec);

    // increase free block counter
    EnterCriticalSection(&cs);
    (*freeBlockCounter)++;
    LeaveCriticalSection(&cs);
}


static unsigned int __stdcall PlayThread(LPVOID lp) {
    /// get the class instance
    CWaveBox *wb = (CWaveBox *) lp;
    CDuration dur;

    /// pooling variables < most frequently used / checked >
    register    WMsg            wmsg = WMSG_WAIT;
    wb->TMSG = TMSG_ALIVE;         
    unsigned int    i = 0;
    dura1.Start();
    /*  for(unsigned int j = 0; j < SUPPORT_INTERFACES; j++)
        /// check for free interface
        if(wb->INTERF[j].state == INT_FREE) {
            /// attach wave to interface
            if(wb->AddInterface(&wb->INTERF[j].dev, &wb->W[j].wfx, &wb->INTERF[j].wfreeblock))
            {
                /// get wave pointer
                wb->INTERF[j].wave = &wb->W[j];
                wb->INTERF[j].state = INT_FREE;
            }
        }
    */

    /// thread life cycle
    while(wb->TMSG) {
        /// check for 'play' msg
        i = wb->wwrite;
        /// read msg
        EnterCriticalSection(&cs);
        wmsg = wb->W[i].WMSG;
        LeaveCriticalSection(&cs);

        /// playable wave
        if(wmsg == WMSG_START) {
            /// link with first free interface
            for(unsigned int j = 0; j < SUPPORT_INTERFACES; j++)

                /// check for free interface
                if(wb->INTERF[j].state == INT_FREE) {
                    /// attach wave to interface
                    if(wb->AddInterface(&wb->INTERF[j].dev, &wb->W[i].wfx, &wb->INTERF[j].wfreeblock)) {
                        /// get wave pointer
                        wb->INTERF[j].wave = &wb->W[i];

                        /// mark interface as used
                        wb->INTERF[j].state = INT_USED;

                        /// free wave 
                        EnterCriticalSection(&cs);
                        wb->W[i].WMSG = WMSG_PLAY;  
                        wb->W[i].sizeLeft = wb->W[i].size;
                        LeaveCriticalSection(&cs);                        

                        /// leave loop
                        break;
                    }
                }
            dura1.Stop();
            TRACE(_T("PlayThread Dure %g"), dura1.GetDuration() / 1000.0);
            dura1.Start();
        }

        /////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////
        ///
        ///                       < main playing loop >
        ///
        ///    search for the first marked interface and play attached wave
        ///

        for(unsigned int k = 0; k < SUPPORT_INTERFACES; k++) {
            /// nothing to do with free interface
            if(wb->INTERF[k].state == INT_FREE)
                continue;

            //            EnterCriticalSection(&cs);
            int free = wb->INTERF[k].wfreeblock;
            //            LeaveCriticalSection(&cs);

            /// nothing to do with full queued interface
            if(free < BP_TURN)
                continue; 

            WAVEHDR *current = NULL;

            /// how much blocks per turn will be queued
            for(unsigned int m = 0; m < BP_TURN; m++) {
                /// set current block pointer
                current = &wb->INTERF[k].wblock[wb->INTERF[k].wcurrblock]; 
                current->dwUser = (DWORD) & wb->W[wb->wwrite];

                // first make sure the header we're going to use is unprepared
                if(current->dwFlags & WHDR_PREPARED)
                    waveOutUnprepareHeader(wb->INTERF[k].dev, current, sizeof(WAVEHDR));     

                /// how much data is left at this interface to play
                unsigned long left = wb->INTERF[k].wave->size - wb->INTERF[k].wpos;
                unsigned long chunk = 0;

                if(left >= BLOCK_SIZE)
                    chunk = BLOCK_SIZE;
                else if(left && left < BLOCK_SIZE)
                    chunk = left;
                else {
                    ////////////////////
                    /// nothing left ///
                    ////////////////////////////////////////////////////////////////////////
                    ///
                    ///                   < clean job, close waveOutProc threads >
                    /// 
                    ///                    all buffers are queued to the interface
                    ///

                    /// get free block count
                    //EnterCriticalSection(&cs);
                    int free = wb->INTERF[k].wfreeblock;
                    //LeaveCriticalSection(&cs);


                    if(free == BLOCK_COUNT)   /// are all blocks played!?
                    {
                        /// unprepare any blocks that are still prepared
                        for(int i = 0; i < wb->INTERF[k].wfreeblock; i++)
                            if(wb->INTERF[k].wblock[i].dwFlags & WHDR_PREPARED)
                                waveOutUnprepareHeader(wb->INTERF[k].dev, &wb->INTERF[k].wblock[i], sizeof(WAVEHDR));

                        if(wb->W[wb->wwrite].sizeLeft == 0 && wb->W[wb->wwrite].WMSG == WMSG_PLAY) {
                            //EnterCriticalSection(&cs);
                            wb->W[wb->wwrite].WMSG = WMSG_WAIT;
                            wb->wwrite++;
                            wb->wwrite %= SUPPORT_WAVES;
                            if(waveCount == SUPPORT_WAVES)
                                SetEvent(ThreadWaitEvent);
                            waveCount--;
                            //LeaveCriticalSection(&cs);
                        }

                        /// close interface
                        if(wb->RemoveInterface(wb->INTERF[k].dev)) {
                            /// free interface
                            wb->INTERF[k].wcurrblock = 0;
                            wb->INTERF[k].state = INT_FREE;
                            wb->INTERF[k].wpos = 0;
                            wb->INTERF[k].wave = NULL;
                        }
                    }

                    /// step out
                    break;
                }
                wb->W[i].sizeLeft -= chunk;
                /// prepare current wave data block header
                memcpy(current->lpData, &wb->INTERF[k].wave->data[wb->INTERF[k].wpos], chunk);

                current->dwUser = (DWORD) & wb->W[i];

                current->dwBufferLength = chunk;   // sizeof block
                wb->INTERF[k].wpos += chunk;   // update position

                /// prepare for playback
                waveOutPrepareHeader(wb->INTERF[k].dev, current, sizeof(WAVEHDR));

                /// push to the queue
                waveOutWrite(wb->INTERF[k].dev, current, sizeof(WAVEHDR));

                /// decrease free block counter
                EnterCriticalSection(&cs);
                wb->INTERF[k].wfreeblock--;
                LeaveCriticalSection(&cs);

                /// point to the next block
                wb->INTERF[k].wcurrblock++;
                wb->INTERF[k].wcurrblock %= BLOCK_COUNT;
            }/// block(s)
        }/// interface(s)

        /// wait 10 ms < save CPU time >
        Sleep(10);
    }/// thread



    ///////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////
    ///
    ///            force to close interfaces which are still playing 
    ///

    for(int i1 = 0; i1 < SUPPORT_INTERFACES; i1++)
        if(wb->INTERF[i1].state == INT_USED)
            if(waveOutReset(wb->INTERF[i1].dev) == MMSYSERR_NOERROR)
                wb->RemoveInterface(wb->INTERF[i1].dev);


    return  THREAD_EXIT; /// return exit code < destructor >
}

