#ifndef WAVEBOX_H
#define WAVEBOX_H

/// precompiler
#include <windows.h>
#include <mmsystem.h>

/// messages
typedef unsigned int WMsg;   // wave messages
typedef unsigned int TMsg;   // thread messages

#define WMSG_WAIT               0       // wave wait
#define WMSG_START              1       // wave play
#define WMSG_PLAY               2       // wave playing
#define TMSG_ALIVE              1       // thread alive
#define TMSG_CLOSE              0       // thread close
#define INT_FREE                0       // interface free
#define INT_USED                1       // interface used
#define THREAD_EXIT     0xABECEDA       // thread exit code

/// performance predefines
#define SUPPORT_WAVES          5       // predefined wave count
#define SUPPORT_INTERFACES     2       // predefined interface count

#define BLOCK_SIZE           1024       // queue block size
#define BLOCK_COUNT          20       // queue block count
#define BP_TURN              4       // blocks per turn


static  CRITICAL_SECTION cs;     // critical section

static unsigned int __stdcall PlayThread(LPVOID lp);                // main play thread
static void CALLBACK waveOutProc(HWAVEOUT  hWaveOut, // waveOut prototype
                                              UINT      uMsg, DWORD     dwInstance, DWORD     dwParam1,
                                 DWORD     dwParam2);
struct WAVE {
        char *data;      /// wave    
        unsigned long size;      /// size
        WAVEFORMATEX wfx;       /// wfx
        WMsg WMSG;      /// { 0,1 } wait / play
        int sizeLeft;
        int *awrite;
};

struct OUTINTERFACE {
        /// interface
        HWAVEOUT dev;        /// device handle
        unsigned int state;      /// { 0,1 } free / used

        /// wave
        WAVE *wave;       /// current wave

        /// wave interface
        unsigned long wpos;       /// current play position
        WAVEHDR *wblock;     /// wave block
        volatile int wfreeblock; /// free blocks left
        int wcurrblock; /// current block
};

class CWaveBox {
    public:

        /// members
        OUTINTERFACE INTERF[SUPPORT_INTERFACES];                      // interface(s)
        WAVE W[SUPPORT_WAVES];                           // wave(s)
        unsigned int wload;                                      // current wave(s) loaded
        TMsg TMSG;                                       // thread msg { 1,0 } alive / close

        /// prototypes 
        int Load(int nSamplerate, int nNumchannels, int nBitspersamp, int bufferlenms, int prebufferms);                              // load wave into WaveBox
        void SetFormat(WAVEFORMATEX *format, unsigned int channels, unsigned int nSamplerate, unsigned int nBitspersamp);
        int Play(char *buf, int len);                              // play n wave from WaveBox ( starts play thread )

        CWaveBox();                                        // play thread created suspended
        virtual ~CWaveBox();                                        // play thread terminated

        /// waveOut open & close
        int AddInterface(HWAVEOUT *dev, /// push wave wfx to interface
                              WAVEFORMATEX *wfx, volatile int *wfreeblock);      
        int RemoveInterface(HWAVEOUT dev);                /// pull wave wfx from interface
        int wwrite;
        int GetOutputTime();
        int GetWrittenTime();
        int CanWrite();
        int IsPlaying();
        int OutputPause(int flag);
        void OutputSetVolume(int volume);
        void OutputSetPan(int pan);

    protected:

        /// thread
        HANDLE thread;                                     /// play thread handle
        unsigned int run;                                        /// suspended / resumed

        /// prototypes
        WAVEHDR *allocateBlocks(int size, int count);          /// alloc heap for wave header blocks
        void freeBlocks(WAVEHDR *blockArray);              /// free heap
};

#endif