/*********************************************************************

	Copyright (C) 2002 Smaller Animals Software, Inc.

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgment in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source distribution.

	  http://www.smalleranimals.com
	  smallest@smalleranimals.com
			  
**********************************************************************/

/**********************************************************************

	These functions define the core operations for the langauge.
	You can change these if you like. 

	If you want to add more operators, you should add them in 
	SASrcEvalFuncsEx.cpp.

	Notes:
	1. Operators are passed in a CSAScrOpStack. Parameters are stored
	in right to left order : 

		&operator(p1, p2, p3)

		p3 = PopText(stack)
		p2 = PopText(stack)
		p1 = PopText(stack)

	2. Everything, every return and parameter, is a string
			
**********************************************************************/

#include "stdafx.h" 
#include "SAScrDefs.h"
#include "SAScrUtils.h"

#include "SAScrEval.h"
#include "SAScrException.h"

//////////////////////////////////////////////////////////////////////////

CSAString __operUpper(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam1 = PopText(paramStack);
	
	
	csParam1.MakeUpper();
	
	csResult = csParam1;
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operLower(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam1 = PopText(paramStack);
	
	
	csParam1.MakeLower();
	
	csResult = csParam1;
	
	return csResult;
}


//////////////////////////////////////////////////////////////////////////

CSAString __operImport(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam1 = PopText(paramStack);
	
	if (!SAScr::ReadTextFile(csResult, csParam1))
   {
      return _T( "Unable to read the file: " ) + csParam1;
   }
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operConcat(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	csResult = csParam1 + csParam2;
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operLeft(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if (csParam2!=_T( "" ))
	{
		int iLen = max(0, _ttoi(csParam2));
		csResult = csParam1.Left(iLen);
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operRight(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if (csParam2!=_T( "" ))
	{
		int iLen = max(0, _ttoi(csParam2));
		csResult = csParam1.Right(iLen);
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operMid(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam3 = PopText(paramStack);
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if ((csParam2!=_T( "" )) || (csParam3!=_T( "" )))
	{
		int iPos = max(0, _ttoi(csParam2));
		int iLen = max(0, _ttoi(csParam3));
		
		csResult = csParam1.Mid(iLen, iPos);
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operReplace(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam3 = PopText(paramStack);
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if ((csParam2!=_T( "" )) || (csParam3!=_T( "" )))
	{
		csResult = SAScr::ReplaceNoCase(csParam1, csParam2, csParam3);
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operLen(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam1 = PopText(paramStack);
	
	
	csResult.Format(_T( "%d" ), csParam1.GetLength());
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operFind(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	csParam2.MakeUpper();
	csParam1.MakeUpper();
	
	int iPos = csParam1.Find(csParam2);
	csResult.Format(_T( "%d" ), iPos);
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operSub(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	double p1 = _tcstod(csParam1, NULL);
	double p2 = _tcstod(csParam2, NULL);
	
	csResult.Format(_T( "%d" ), (int)(p1 - p2));
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operAdd(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	double p1 = _tcstod(csParam1, NULL);
	double p2 = _tcstod(csParam2, NULL);
	
	csResult.Format(_T( "%d" ), (int)(p1 + p2));
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operMul(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	double p1 = _tcstod(csParam1, NULL);
	double p2 = _tcstod(csParam2, NULL);
	
	csResult.Format(_T( "%d" ), (int)(p1 * p2));
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operDiv(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	double p1 = _tcstod(csParam1, NULL);
	double p2 = _tcstod(csParam2, NULL);
	
	if (p2!=0)
	{
		csResult.Format(_T( "%d" ), (int)(p1 / p2));
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operIf(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_ERROR;
	
	CSAString csParam3 = PopText(paramStack);
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	bool bTrue = SAScr::OperandToBool(csParam1);
	
	if (bTrue)
	{
		csResult = csParam2;
	}
	else
	{
		csResult = csParam3;
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operEql(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	// do Eql, then flip if we're doing NotEql
	csResult= SA_VAR_FALSE;
	
	if (csParam1.CompareNoCase(csParam2)==0)
	{
		csResult= SA_VAR_TRUE;
	}
	else
	{
		if (SAScr::IsAllDigits(csParam1) && SAScr::IsAllDigits(csParam2))
		{
			if (_tcstod(csParam1, NULL) == _tcstod(csParam2, NULL))
			{
				csResult=SA_VAR_TRUE;
			}
		}
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operNEql(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	
	CSAString csParam1 = PopText(paramStack);
	
	
	// do Eql, then flip if we're doing NotEql
	csResult= SA_VAR_FALSE;
	
	if (csParam1.CompareNoCase(csParam2)!=0)
	{
		csResult= SA_VAR_TRUE;
	}
	else
	{
		if (SAScr::IsAllDigits(csParam1) && SAScr::IsAllDigits(csParam2))
		{
			if (_tcstod(csParam1, NULL) != _tcstod(csParam2, NULL))
			{
				csResult=SA_VAR_TRUE;
			}
		}
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operLEql(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if (SAScr::IsAllDigits(csParam1) && SAScr::IsAllDigits(csParam2))
	{
		if (_tcstod(csParam1, NULL) <= _tcstod(csParam2, NULL))
		{
			csResult = SA_VAR_TRUE;
		}
	}
	else
	{
		if (csParam1 <= csParam2)
		{
			csResult = SA_VAR_TRUE;
		}
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operGEql(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if (SAScr::IsAllDigits(csParam1) && SAScr::IsAllDigits(csParam2))
	{
		if (_tcstod(csParam1, NULL) >= _tcstod(csParam2, NULL))
		{
			csResult = SA_VAR_TRUE;
		}
	}
	else
	{
		if (csParam1 >= csParam2)
		{
			csResult = SA_VAR_TRUE;
		}
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operGrtr(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if (SAScr::IsAllDigits(csParam1) && SAScr::IsAllDigits(csParam2))
	{
		if (_tcstod(csParam1, NULL) > _tcstod(csParam2, NULL))
		{
			csResult = SA_VAR_TRUE;
		}
	}
	else
	{
		if (csParam1 > csParam2)
		{
			csResult = SA_VAR_TRUE;
		}
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operLess(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	if (SAScr::IsAllDigits(csParam1) && SAScr::IsAllDigits(csParam2))
	{
		if (_tcstod(csParam1, NULL) < _tcstod(csParam2, NULL))
		{
			csResult = SA_VAR_TRUE;
		}
	}
	else
	{
		if (csParam1 < csParam2)
		{
			csResult = SA_VAR_TRUE;
		}
	}
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operNot(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam1 = PopText(paramStack);
	
	bool bTrue = SAScr::OperandToBool(csParam1);
	
	csResult = (bTrue ? SA_VAR_FALSE : SA_VAR_TRUE);
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operOr(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	
	bool bTrue = SAScr::OperandToBool(csParam1) || SAScr::OperandToBool(csParam2);
	
	csResult = (bTrue ? SA_VAR_TRUE : SA_VAR_FALSE);
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operAnd(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	bool bTrue = SAScr::OperandToBool(csParam1) && SAScr::OperandToBool(csParam2);
	
	csResult = (bTrue ? SA_VAR_TRUE : SA_VAR_FALSE);
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operRand(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
	CSAString csResult = SA_VAR_FALSE;
	
	CSAString csParam2 = PopText(paramStack);
	
	CSAString csParam1 = PopText(paramStack);
	
	int i1 = _ttoi(csParam1);
	int i2 = _ttoi(csParam2);
	int iMax = max(i1, i2);
	int iMin = min(i1, i2);
	
	csResult.Format(_T( "%d" ), (rand() % ((iMax - iMin) + 1)) + iMin);
	
	return csResult;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operEcho(CSAScrOpStack &paramStack, const CSAScrOperator& /*oper*/)
{
   CSAString csParam1 = PopText(paramStack);

	return csParam1;
}

//////////////////////////////////////////////////////////////////////////

CSAString __operVarSet(CSAScrOpStack& /*paramStack*/, const CSAScrOperator& /*oper*/)
{
	return _T( "" );
}

//////////////////////////////////////////////////////////////////////////

bool SAScr::OperandToBool(const CSAString &csOperand)
{
	if (csOperand.IsEmpty())
	{
		return false;
	}
	else
	{
		if (SAScr::IsAllDigits(csOperand))
		{
			if (_tcstod(csOperand, NULL)==0.0)
			{
				return false;
			}
		}
	}
	
	return true;
}

//////////////////////////////////////////////////////////////////////////

CSAString PopText(CSAScrOpStack &paramStack)
{
	CSAScrOp oper = paramStack.top();
	CSAString csRet = oper.m_text;
	
	paramStack.pop();
	
	return csRet;
}

