/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/

//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SAScrString.h"
#include "SAScrException.h"

//////////////////////////////////////////////////////////////////////
// error strings - change these if you want
#define PARSE_SYNTAX_MSG         _T( "syntax error" )
#define PARSE_INTERNAL_MSG       _T( "fatal internal error" )
#define PARSE_RECURSIONLIMIT_MSG _T( "internal recursion limit reached" )
#define PARSE_UNKNOWN_SYMBOL_MSG _T( "unrecognized variable or operator" )
#define PARSE_NONTERMEXP_MSG     _T( "invalid expression (no end found)" )
#define PARSE_PARAMETERCOUNT_MSG _T( "incorrect number of parameters" )

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSAScrException::CSAScrException()
{
   m_eType = eParseNone;
}

//////////////////////////////////////////////////////////////////////

CSAScrException::~CSAScrException()
{

}

//////////////////////////////////////////////////////////////////////

CSAString CSAScrException::Text()
{
   CSAString t;
   if (m_csText.GetLength() > 0)
   {
      t.Format(_T( "[%s: %s]" ), TranslateError(m_eType), m_csText);
   }
   else
   {
      t.Format(_T( "[%s]" ), TranslateError(m_eType));
   }

   return t;
}

//////////////////////////////////////////////////////////////////////

CSAString CSAScrException::TranslateError(eParseError eError)
{
   CSAString csOut;
   switch (eError)
   {
   default:
   case eParseInternal:
      csOut = PARSE_INTERNAL_MSG;
      break;
   case eParseSyntax:
      csOut = PARSE_SYNTAX_MSG;
      break;
   case eParseRecursionLimit:
      csOut = PARSE_RECURSIONLIMIT_MSG;
      break;
   case eParseUnknownSymbol:
      csOut = PARSE_UNKNOWN_SYMBOL_MSG;
      break;
   case eParseSyntaxNonterm:
      csOut = PARSE_NONTERMEXP_MSG;
      break;
   case eParseParameterCount:
      csOut = PARSE_PARAMETERCOUNT_MSG;
      break;
   }
   
   return csOut;
}
