/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/


#include "stdafx.h"
#include "SAScrMacros.h"
#include "SAScrTokenizer.h"
#include "SAScrUtils.h"

CSAMacro::CSAMacro()
{
   Clear();
}

void CSAMacro::Clear()
{
   m_iParamCount = -1;

   m_csName.Empty();
   m_csParams.Empty();
   m_csFunc.Empty();
}

CSAMacro::CSAMacro(const TCHAR	* name, const TCHAR	* paramList, int paramCount, const TCHAR	* functionText) 
{
   m_csName = name;
   m_csParams = paramList;
   m_csFunc = functionText;
   m_iParamCount = paramCount;
}


////////////////////////////////////////////////////////////////
#ifdef _DEBUG
void CSAMacro::Dump() const
{
   SAScr::SAScrTrace(_T( "%s(%s) %s\n" ), m_csName, m_csParams, m_csFunc);
}
#endif

////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void CSAMacroMap::Dump() const
{
   for (const_iterator it=begin(); it!=end(); it++)
   {
      (*it).second.Dump();
   }
}
#endif

////////////////////////////////////////////////////////////////

void CSAMacroMap::Add(const CSAMacro & m)
{
   CSAString csName(m.m_csName);
   csName.MakeUpper();

   insert(value_type(csName, m));
}

////////////////////////////////////////////////////////////////

void CSAMacroMap::Add(const TCHAR	*pName, const TCHAR	*pParams, const TCHAR	*pFunc, int iParams)
{
   CSAMacro m(pName, pParams, iParams, pFunc);

   CSAString csName(pName);
   csName.MakeUpper();

   insert(value_type(csName, m));
}

//////////////////////////////////////////////////////////////////////////

bool operator < (const CSAMacro &a, const CSAMacro &b)
{
   return a.m_csName < b.m_csName;
}

//////////////////////////////////////////////////////////////////////

bool operator == (const CSAMacro &a, const CSAMacro &b)
{
   return (a.m_csName.CompareNoCase(b.m_csName) == 0);
}

//////////////////////////////////////////////////////////////////////////
// this can be used to serialize a CSAMacro
CSAString	CSAMacro::Pack()
{
	CSAString csOut;

	csOut.Format(_T( "%s%c%d%c%s%c%s%c" ), m_csName, packseparator, m_iParamCount, packseparator, m_csParams, packseparator, m_csFunc, packseparator);

	return csOut;
}

//////////////////////////////////////////////////////////////////////////

void CSAMacro::UnPack(const TCHAR	*PTSTR)
{
   m_iParamCount = 0;
   m_csName = m_csParams = m_csFunc = _T( "" );
	
	CSAString csTemp(PTSTR);

	int iTagCount = 0;
	CSAString csCurTag;
	int iLen = csTemp.GetLength();

	for (int i=0; i<iLen; i++)
	{
		if (csTemp.GetAt(i)!=packseparator)
		{
			csCurTag+=csTemp.GetAt(i);
		}
		else
		{
			switch (iTagCount)
			{
			case 0:
				m_csName = csCurTag;
				break;
			case 1:
				m_iParamCount = _ttoi(csCurTag);
				break;
			case 2:
				m_csParams = csCurTag;
				break;
			case 3:
				m_csFunc = csCurTag;
				break;
			}

			csCurTag=_T( "" );
			iTagCount++;
			continue;
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// break the csParams string into an array of parameter names
void     CSAMacro::GetFormalParamArray(const CSAString &csParams, CSAStringArray &csaPTokens)
{
	CSAString csCur = csParams;
	csaPTokens.RemoveAll();
	
	int iCurPos = 0;
	int iSepPos = csParams.Find(_T( ',' ));
	
	if (iSepPos==-1)
	{
		if (!csCur.IsEmpty())
		{
			csaPTokens.Add(csCur);
		}
	}
	
	while (iSepPos!=-1)
	{
		CSAString csCurToken = csCur.Mid(iCurPos, iSepPos - iCurPos);
		csCurToken.TrimLeft();
		csCurToken.TrimRight();
		
		csaPTokens.Add(csCurToken);
		
		iCurPos = iSepPos + 1;
		
		csCur = csCur.Mid(iCurPos);
		csCur.TrimLeft();
		csCur.TrimRight();
		
		iSepPos = csCur.Find(_T( ',' ));
		if (iSepPos==-1)
		{
			csaPTokens.Add(csCur);
		}
	}
}
