/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#ifndef VARMACROSH
#define VARMACROSH

#pragma warning(disable:4786)

#include <map>

using namespace std ;
using namespace std::rel_ops ;

#include "SAScrStringArray.h"

////////////////////////////////////////////////////////////////

struct CSAMacro
{
   CSAMacro();

   // ex. CSAMacro t("__min", "__p1, __p2", 2, "&If(Grtr(__value1, __value2), __value1, __value2)");
   CSAMacro(const TCHAR * name, const TCHAR * paramList, int paramCount, const TCHAR * functionText);

   // pack this structure into a CSAString - for serialization
	CSAString	Pack();

   // pack this structure from a CSAString - for de-serialization
	void		UnPack(const TCHAR *);

   // initialize
   void     Clear();

   // break a CSAMacro into tokens
   static void     GetFormalParamArray(const CSAString &csParams, CSAStringArray &csaPTokens);

#ifdef _DEBUG
	void Dump() const;
#else
	_inline void Dump() const {}
#endif
public:
   int m_iParamCount;

   CSAString m_csName;
   CSAString m_csParams;
   CSAString m_csFunc;

   //
   //	[JRT]	If This Is A "TCHAR", Lets Use A TCHAR...
   //
//   enum {packseparator=(TCHAR)0xfe};
   enum {packseparator='\fe'};
};

////////////////////////////////////////////////////////////////

class CSAMacroMap : public map <CSAString, CSAMacro>
{
public:
   void Add(const CSAMacro &);

   void Add(const TCHAR *pName, const TCHAR *pParams, const TCHAR *pFunc, int iParams);

#ifdef _DEBUG
	void Dump() const;
#else
	_inline void Dump() const {}
#endif

};

////////////////////////////////////////////////////////////////

extern bool operator < (const CSAMacro &a, const CSAMacro &b);
extern bool operator == (const CSAMacro &a, const CSAMacro &b);

#endif


