/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com
			  
**********************************************************************/
#include "stdafx.h"
#include "SAScrOperManager.h" 
#include "SAScrDefs.h" 
#include "SAScrEvalFuncDefs.h" 
#include "SAScrEval.h" 
#include "SAScrEvalFuncsEx.h" 
#include "time.h" 

// the one and only operator manager. 
// this must be initialized prior to calling any parsing functions
CSAScrOpManager g_varOperManager;

/////////////////////////////////////////////////////////////////////////////
// Names for the core built-in operators.
// if you want to add new functions, add them in SAScrEvalFuncsEx.cpp/.h

const TCHAR	* g_operUpper	= _T( "Upper" );
const TCHAR	* g_operLower	= _T( "Lower" );

const TCHAR	* g_operConcat	= _T( "Join" );
const TCHAR	* g_operLeft    = _T( "Left" );
const TCHAR	* g_operRight   = _T( "Right" );
const TCHAR	* g_operMid     = _T( "Mid" );
const TCHAR	* g_operReplace = _T( "Repl" );
const TCHAR	* g_operLen     = _T( "Len" );
const TCHAR	* g_operFind    = _T( "Find" );

const TCHAR	* g_operAdd     = _T( "Add" );
const TCHAR	* g_operSub     = _T( "Sub" );
const TCHAR	* g_operMul     = _T( "Mul" );
const TCHAR	* g_operDiv     = _T( "Div" );

const TCHAR	* g_operIf      = _T( "If" );
const TCHAR	* g_operOr      = _T( "Or" );
const TCHAR	* g_operAnd     = _T( "And" );

const TCHAR	* g_operEql     = _T( "Eql" );
const TCHAR	* g_operNotEql  = _T( "NEql" );
const TCHAR	* g_operNot     = _T( "Not" );
const TCHAR	* g_operLess    = _T( "Less" );
const TCHAR	* g_operGreater = _T( "Grtr" );
const TCHAR	* g_operLessEql = _T( "LEql" );
const TCHAR	* g_operGrtrEql = _T( "GEql" );

const TCHAR	* g_operRand    = _T( "Rand" );
const TCHAR	* g_operImport	= _T( "Import" );

const TCHAR	* g_operEcho	= _T( "T" );

const TCHAR	* g_operVarSet	= _T( "VarSet" );

//////////////////////////////////////////////////////////////////////
// name, function, param count, param name string, function name, function description
// if you want to add new functions, add them in SAScrEvalFuncsEx.cpp/.h

CSAScrOperator g_operArray[] = 
{
	CSAScrOperator(g_operUpper,	__operUpper,             1 ,_T( "Text" ) ,      _T( "" ), _T( "Convert \"Text\" to upper case" ) ),
	CSAScrOperator(g_operLower,	__operLower,             1 ,_T( "Text" ) ,      _T( "" ), _T( "Convert \"Text\" to lower case" ) ),
	CSAScrOperator(g_operLeft,		__operLeft,              2 ,_T( "Text, n" ),    _T( "" ), _T( "Leftmost 'n' characters of \"Text\"" ) ),
	CSAScrOperator(g_operRight,	__operRight,             2 ,_T( "Text, n" ),    _T( "" ), _T( "Rightmost 'n' characters of \"Text\"" ) ),
	CSAScrOperator(g_operMid,		__operMid,               3 ,_T( "Text, i, n" ), _T( "" ), _T( "First 'i' characters of \"Text\", starting at character 'n'" ) ),
	CSAScrOperator(g_operReplace,	__operReplace,           3 ,_T( "Text, t1, t2" ), _T( "" ), _T( "Replace all occurances of \"t1\" in \"Text\" with \"t2\" (case insensitive)" ) ),
	CSAScrOperator(g_operLen,   	__operLen,               1 ,_T( "Text" ),       _T( "" ), _T( "Text Length" )),
	CSAScrOperator(g_operFind,   	__operFind,              2 ,_T( "t1, t2" ),     _T( "" ), _T( "Find text \"t2\" in text \"t1\". Returns position of t2 in t1 (starting at 0), else -1." ) ),
	CSAScrOperator(g_operImport,	__operImport,            1 ,_T( "Path" ) ,      _T( "" ), _T( "Import text from file \"Path\"" ) ),
	CSAScrOperator(g_operConcat,	__operConcat,            2 ,_T( "t1, t2" )	,  _T( "" ), _T( "Concatenate text \"t1\" and \"t2\"" ) ),
	
	CSAScrOperator(g_operAdd,		__operAdd,               2, _T( "x, y" ) ,      _T( "" ), _T( "Add x and y" )),
	CSAScrOperator(g_operSub,		__operSub,               2, _T( "x, y" ) ,      _T( "" ), _T( "Subtract y from x" )),
	CSAScrOperator(g_operMul,		__operMul,               2, _T( "x, y" ) ,      _T( "" ), _T( "Multiply x and y" )),
	CSAScrOperator(g_operDiv,		__operDiv,               2, _T( "x, y" ) ,      _T( "" ), _T( "Divide x by y" )),
	
	CSAScrOperator(g_operRand,		__operRand,              2, _T( "x, y" ) ,      _T( "" ), _T( "Generate a random number between x and y" )),
	
	CSAScrOperator(g_operIf,		__operIf,                3 ,_T( "x, A, B" ) ,   _T( "" ), _T( "If x does not equal 0 or is non-empty, output A, else B" )),
	CSAScrOperator(g_operOr,		__operOr,                2 ,_T( "x, y" ) ,      _T( "" ), _T( "If one of x, y is not equal to 0 or is non-empty, output \"1\", else \"0\"" ) ),
	CSAScrOperator(g_operAnd,		__operAnd,               2 ,_T( "x, y" ) ,      _T( "" ), _T( "If both x and y are not equal to 0 and non-empty, output \"1\", else \"0\"" ) ),
	
	CSAScrOperator(g_operEql,		__operEql,               2 ,_T( "x, y" ) ,      _T( "" ), _T( "If x = y, output \"1\" , else \"0\"" ) ),
	CSAScrOperator(g_operNotEql,	__operNEql,              2 ,_T( "x, y" ) ,      _T( "" ), _T( "If x = y, output \"0\" , else \"1\"" ) ),
	CSAScrOperator(g_operNot,		__operNot,               1 ,_T( "x" )    ,      _T( "" ), _T( "If x equals 0 or is empty, output \"1\", else \"0\"" ) ),
	CSAScrOperator(g_operLess,		__operLess,              2 ,_T( "x, y" ) ,      _T( "" ), _T( "If x is less than y, output \"1\", else \"0\"" ) ),
	CSAScrOperator(g_operGreater,	__operGrtr,              2 ,_T( "x, y" ) ,      _T( "" ), _T( "If x is greater than y, output \"1\", else \"0\"" ) ),
	CSAScrOperator(g_operLessEql,	__operLEql,              2 ,_T( "x, y" ) ,      _T( "" ), _T( "If x is less than or equal to y, output \"1\", else \"0\"" ) ),
	CSAScrOperator(g_operGrtrEql,	__operGEql,              2 ,_T( "x, y" ) ,      _T( "" ), _T( "If x is greater than or equal to y, output \"1\", else \"0\"" ) ),

	CSAScrOperator(g_operEcho,		__operEcho,              1 ,_T( "Text" ) ,      _T( "" ), _T( "Output \"Text\"" ) ),

	CSAScrOperator(g_operVarSet,	__operVarSet,            2 ,_T( "\"Var\", Val" ),      _T( "" ), _T( "Create a variable, \"Var\", with value \"Val\". Variable name must be in quotes." ) ),
};

/////////////////////////////////////////////////////////////////////////////

// built in variable names

const TCHAR	* g_varNULL  	= _T( "noChar" );

//////////////////////////////////////////////////////////////////////
// built in variables
// name, description, value, context

const CSAScrVariable g_varArray[] = 
{
	CSAScrVariable(g_varNULL,    _T( "Empty text (length = 0)" ),       _T( "" ),   CTXALL),
};

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

CSAScrOpManager::CSAScrOpManager()
{
	m_bMapsInit = false;
}

//////////////////////////////////////////////////////////////////////

bool CSAScrOpManager::Initialize(CSAMacroMap &macros, CSAScrVariableMap &vars)
{
	m_bMapsInit = true;
	
	InitVariableMap(vars);
	
	InitOperatorMap(macros);
	
	return true;
}

//////////////////////////////////////////////////////////////////////

CSAScrOpManager::~CSAScrOpManager()
{
}

//////////////////////////////////////////////////////////////////////

void CSAScrOpManager::InitVariableMap(CSAScrVariableMap &userVariableMap)
{
	m_variableMap.clear();
	
	// core language vars
	for (int i=0; i< ARRAYSIZE(g_varArray); i++)
	{
		m_variableMap.Add(g_varArray[i]);
	}
	
	// extra vars
	CSAScrVariableMap::iterator theIterator;
	for (theIterator=userVariableMap.begin(); theIterator!=userVariableMap.end(); theIterator++)
	{
		m_variableMap.Add((*theIterator).second);
	}
}

//////////////////////////////////////////////////////////////////////

void CSAScrOpManager::InitOperatorMap(CSAMacroMap &userMacroMap)
{
	// for Rand
	srand((int)time(NULL));
	
	m_operatorMap.clear();
	

	for (int i=0; i<ARRAYSIZE(g_operArray); i++)
	{
#ifndef SASCR_SILENT
		g_operArray[i].Dump();
#endif
		m_operatorMap.Add(g_operArray[i]);
	}
	
	// add extension built-in operators
	InitOperatorMapEx(m_operatorMap);
	
   m_operatorMap.AddMacros(userMacroMap);

}



