/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#include "stdafx.h"
#include "SAScrString.h"
#include "SAScrUtils.h"

#ifndef USE_MFC_CSTRING

// this is a minimal CString replacement class

CSAString::CSAString()
{
}

CSAString::CSAString(const CSAString& s)
{
   if (s!=*this)
   {
      m_str.erase();
	   m_str.assign(s.m_str.c_str());
   }
}

CSAString::CSAString(const string & s)
{
   if (s!=m_str)
   {
      m_str.erase();
	   m_str.assign(s.c_str());
   }
}

CSAString::CSAString(const TCHAR	*c)
{
   if (c!=m_str.c_str())
   {
      m_str.erase();
	   m_str.assign(c);
   }
}

int CSAString::GetLength() const
{
	return m_str.size();
}

int CSAString::Compare(const CSAString &x) const
{
	return (strcmp(m_str.c_str(), x.m_str.c_str()));
}

int CSAString::CompareNoCase(const CSAString &x) const
{
   return (sa_strcmpi(m_str.c_str(), x.m_str.c_str()));
}

bool CSAString::IsEmpty() const
{
	return (m_str.size()==0);
}

void CSAString::MakeUpper()
{
	if (m_str.size() > 0)
	{
		sa_strupr(const_cast<TCHAR	*>(m_str.data()));
	}
}

void CSAString::MakeLower()
{
	if (m_str.size() > 0)
	{
		sa_strlwr(const_cast<TCHAR	*>(m_str.data()));
	}
}

CSAString& CSAString::TrimLeft()
{
	const TCHAR	* szTrimChars = _T( " \t" );
	m_str.erase(0, m_str.find_first_not_of(szTrimChars));
	return *this;
}

CSAString& CSAString::TrimRight()
{
	const TCHAR	* szTrimChars = _T( " \t" );
	int nIdx	= m_str.find_last_not_of(szTrimChars);
	if ( m_str.npos == nIdx )
	{
		m_str.erase();
	}
	else
	{
		char tSave	= m_str.at(nIdx);
		m_str.erase(nIdx, m_str.npos);	
		m_str.append(1, tSave);
	}
	return *this;
}

CSAString CSAString::Mid(int iStart, int iCount) const
{			 
   iCount = min((int)m_str.length(), iCount);

   if (iCount==-1)
   {
      iCount = m_str.length();
   }

   iStart = max(0, iStart);

	CSAString ret = m_str.substr(iStart, iCount);
	
	return ret;
}

CSAString CSAString::Left(int nCount) const
{
	CSAString ret = m_str.substr(0, nCount);
	return ret;
}

CSAString CSAString::Right(int nCount) const
{
	nCount = min(nCount, m_str.size());
	return m_str.substr(m_str.size()-nCount);
}


int  CSAString::Find(const char ch)  const
{
	int nIdx	= m_str.find_first_of(ch);
	return (nIdx == m_str.npos ? -1 : nIdx);
}

int  CSAString::Find(const CSAString&fs)  const
{
	int nIdx	= m_str.find(fs);
	return (nIdx == m_str.npos ? -1 : nIdx);
}

int  CSAString::ReverseFind(const char ch)  const
{
	int nIdx	= m_str.find_last_of(ch);
	return (nIdx == m_str.npos ? -1 : nIdx);
}

int  CSAString::FindOneOf(const TCHAR	*szCharSet) const
{
	int nIdx = m_str.find_first_of(szCharSet);
	return (nIdx == m_str.npos ? -1 : nIdx);
}

void CSAString::Format(const char* szFormat, ...)
{
	va_list argList;
	va_start(argList, szFormat);
	FormatV(szFormat, argList);
	va_end(argList);
}

#define MAX_FMT_TRIES		5	 // #of times we try if stack buffers not enough
#define FMT_BLOCK_SIZE		2048 // # of bytes above BUFSIZE_2ND_TRY each try
#define BUFSIZE_1ST	256
#define BUFSIZE_2ND 512
#define STD_BUF_SIZE		1024
void CSAString::FormatV(const char* szFormat, va_list argList)
{
	// We're going to use the  _m_vsntprintf function, assuming FMT_BLOCK_SIZE
	// characters.

	int nTriesLeft			= MAX_FMT_TRIES-1;
	int nUsed				= -1;
	char szBuf1[BUFSIZE_1ST];
	
	if ( (nUsed=_vsnprintf(szBuf1, BUFSIZE_1ST-1, szFormat, argList)) >= 0 )
	{
		szBuf1[nUsed]		= _T( '\0' );
		*this				= szBuf1;
	}
	else
	{
		char szBuf2[BUFSIZE_2ND];
		if ( (nUsed=_vsnprintf(szBuf2, BUFSIZE_2ND-1, szFormat, argList))>=0)
		{
			szBuf2[nUsed]	= _T( '\0' );
			*this			= szBuf2;
		}
		else
		{
			int nChars		= BUFSIZE_2ND; 

			// Loop until we succeed or we have exhausted the number of tries

			do	
			{
				nChars		+= FMT_BLOCK_SIZE;	// # of TCHARS in the string

				// Now try the actual formatting. 

				nUsed		= _vsnprintf(Buffer(nChars+1),
										nChars,
										szFormat,
										argList);

				if ( nUsed >= 0 )
					m_str.resize(nUsed);

			} while ( nUsed < 0 && --nTriesLeft > 0);
		}
	}
}

char* CSAString::Buffer(int nMinLen)
{
	if ( nMinLen > -1 )
		m_str.resize((int)(nMinLen));

	return m_str.empty() ? const_cast<TCHAR	*>(m_str.data()) : &(m_str.at(0));
}

CSAString& CSAString::operator=(const TCHAR	*src)
{
	m_str.erase();

	if (src!=NULL)
	{
		m_str.assign(src);
	}

	return *this;
}

CSAString& CSAString::operator=(const CSAString & src)
{
	m_str.erase();

	if (src!=NULL)
	{
		m_str.assign(src.m_str.c_str());
	}

	return *this;
}

CSAString& CSAString::operator=(string & src)
{
	m_str.erase();

	m_str.assign(src.c_str());

	return *this;
}

bool CSAString::operator ==(const CSAString &in) const
{
   return Compare(in) == 0;
}

bool CSAString::operator ==(const TCHAR	*pIn) const
{
   return Compare(pIn) == 0;
}

bool CSAString::operator ==(char c) const
{
   if (GetLength()!=1)
   {
      return false;
   }

   return GetAt(0) == c;
}

bool CSAString::operator <(const CSAString &in) const
{
   return Compare(in) < 0;
}

CSAString::operator const TCHAR	*() const
{
	return m_str.c_str();
}

CSAString CSAString::operator +(const CSAString &x) const
{
	CSAString n(*this);
	
	n.m_str.append(x);

	return n;
}

CSAString CSAString::operator +(const CSAString &in)
{
   CSAString t;
   t = *this;
   t+=in;

   return t;
}

CSAString CSAString::operator +(const TCHAR	*pIn)
{
   CSAString t;
   t = *this;
   t+=pIn;

   return t;
}

CSAString operator +(const CSAString &sStr, const TCHAR	*cStr)
{
   CSAString t;
   t = sStr;
   t+=cStr;

   return t;
}

CSAString& CSAString::operator +=(const CSAString &x)
{
	m_str.append(x);

	return *this;
}

CSAString& CSAString::operator +=(const TCHAR	*x)
{
	m_str.append(x);

	return *this;
}

CSAString& CSAString::operator +=(const char x)
{
	m_str.append(1, x);

	return *this;
}

char CSAString::GetAt(int n) const
{
	return m_str.at(n);
}

#endif // USE_MFC_CSTRING

//////////////////////////////////////////////////////////////////////

bool CSAString::IsOneOf(const TCHAR	*pChars) const
{
   if (GetLength() != 1)
   {
      return false;
   }

   return (FindOneOf(pChars) != -1);
}

