/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#ifndef VARSTRINGH
#define VARSTRINGH

/*
   If you want to use MFC CStrings, you need to #define USE_MFC_CSTRING, otherwise,
   it will use our std::string variation
*/
#define USE_MFC_CSTRING

#include <string>
using namespace std ;

#ifdef USE_MFC_CSTRING

// simple...
class CSAString : public CString
{
public:
	CSAString() : CString( ){};
	CSAString( const CString& stringSrc) : CString( stringSrc ){};
	CSAString( const CSAString& stringSrc) : CString( stringSrc ){};
	CSAString( TCHAR ch, int nRepeat = 1 ) : CString( ch, nRepeat ){};
	CSAString( LPCTSTR lpch, int nLength ) : CString( lpch, nLength ){};
	CSAString( const unsigned char* psz ) : CString( psz ){};
	CSAString( LPCWSTR lpsz ) : CString( lpsz ){};
	CSAString( LPCSTR lpsz ) : CString( lpsz ){};

public:
   bool IsOneOf(const TCHAR *pChars) const;
};

#else

// write our own
class CSAString
{
public:
	CSAString();
	CSAString(const string &);
   CSAString(const CSAString &);
	CSAString(const TCHAR *);

	int GetLength() const;
	int Compare(const CSAString &) const;
	int CompareNoCase(const CSAString &) const;
	
   bool IsEmpty() const;
	
   void Empty() {m_str.erase();}
   void MakeUpper();
	void MakeLower();
	
   CSAString& TrimLeft();				 
	CSAString& TrimRight();

	CSAString Mid(int iStart, int iCount=-1) const;
	CSAString Left(int) const;
	CSAString Right(int) const;

	int  Find(const TCHAR)  const;
	int  Find(const CSAString&)  const;
	int  ReverseFind(const TCHAR)  const;
	int  FindOneOf(const TCHAR *) const;
   bool IsOneOf(const TCHAR *pChars) const;

	void Format(const TCHAR* szFormat, ...);	
	
   CSAString& operator=(const TCHAR *);
	CSAString& operator=(string &);
	CSAString& operator=(const string &);
   CSAString& operator=(const CSAString&);
	
   operator const TCHAR *() const;

	CSAString operator +(const CSAString &) const;
   CSAString operator +(const CSAString &);
   friend CSAString operator +(const CSAString &, const TCHAR *);
   CSAString operator +(const TCHAR *);
   
   bool operator < (const CSAString &) const;
   bool operator==(const CSAString &) const;
   bool operator==(const TCHAR *) const;
   bool operator==(TCHAR) const;

	CSAString& operator +=(const CSAString &x);
	CSAString& operator +=(const TCHAR *x);
	CSAString& operator +=(const TCHAR);

	TCHAR GetAt(int) const;
	
   TCHAR* Buffer(int nMinLen);

protected:
   string m_str;

	void  FormatV(const TCHAR* szFormat, va_list argList);
};
#endif // !USE_MFC_CSTRING


#endif