/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#include "stdafx.h"
#include <math.h>
#include <io.h>
#include "SAScrUtils.h"

/////////////////////////////////////////////////////////////////////////////
// These are some general utility functions 

bool SAScr::ReadTextFile(CSAString &csResult, const CSAString &csFQInput)
{
	csResult=_T( "" );

	if (_taccess(csFQInput,0)!=0)
		return false;

   bool ok = true;
	char buf[1025];

	FILE *fp = _tfopen(csFQInput, _T( "r" ));
	if (fp)
	{
		try 
		{
			while (!feof(fp)) 
			{
				unsigned long read = fread(buf, 1, 1024, fp);
				buf[read]=0;
				csResult+=buf;
			}
		}
		catch (...)
		{
         // error
         ok=false;
		}

		fclose(fp);
	}

	return ok;
}


CSAString SAScr::ReplaceNoCase(const TCHAR	*pString, const TCHAR	*pSearch, const TCHAR	*pReplace)
{
   CSAString csOut;

   int iLen = _tcslen(pString);
   int iSearchLen = _tcslen(pSearch);
   //
   //	[JRT]	Not Used...  Feature Or Flaw...?
   //
//   int iReplaceLen = _tcslen(pReplace);

   for (int i=0;i<iLen;i++)
   {
      if (iLen - i >= iSearchLen)
      {
         if (sa_strnicmp((pString + i), pSearch, iSearchLen)==0)
         {
            csOut+=pReplace;
            i+=iSearchLen;
            i--;
         }
         else
         {
            csOut+=pString[i];
         }
      }
      else
      {
         csOut+=pString[i];
      }
   }

   return csOut;
}

//////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void SAScr::SAScrTrace(const TCHAR* fmt, ...)
#else
void SAScr::SAScrTrace(const TCHAR* /* fmt*/, ...)
#endif
{
#ifdef _DEBUG
	//
	//	[JRT]	The *s.printf(...) Functions May Not Place 
	//	A NUL Terminator If Buffer Length Is Reached...
	//	I Use What I Call The "Plus 1 Rule"...
	//
	TCHAR buffer[1024 + 1];
	va_list argptr;
	va_start(argptr, fmt);
	_vsntprintf(buffer, 1024, fmt, argptr);
	buffer[ 1024 ] = _T( '\0' );
	va_end(argptr); 

	OutputDebugString(buffer);
#endif
}

//////////////////////////////////////////////////////////////////////

bool SAScr::IsAllDigits(const CSAString &cs, bool bCountNegative)
{
   int iLen = cs.GetLength();
   for (int i=0; i<iLen; i++)
   {
      if ((i==0) && bCountNegative)
      {
         if (cs.GetAt(i)==_T( '-' ))
         {
            continue;
         }
      }

      if (!isdigit(cs.GetAt(i)))
      {
         if (cs.GetAt(i)!=_T( '.' ))
         {
            return false;
         }
      }
   }

   return true;
}

//////////////////////////////////////////////////////////////////////

int sa_strnicmp (const TCHAR	* first, const TCHAR	* last, size_t count)
{
   int f,l;

   if ( count )
   {
      do 
      {
        if ( ((f = (unsigned char)(*(first++))) >= _T( 'A' )) && (f <= _T( 'Z' )) )
            f -= _T( 'A' ) - _T( 'a' );

        if ( ((l = (unsigned char)(*(last++))) >= _T( 'A' )) && (l <= _T( 'Z' )) )
            l -= _T( 'A' ) - _T( 'a' );

      } 
      while ( --count && f && (f == l) );
      
      return( f - l );
   }

   return( 0 );
}

//////////////////////////////////////////////////////////////////////

int sa_strcmpi(const TCHAR	* dst, const TCHAR	* src)
{
   int f,l;
   do {
       if ( ((f = (unsigned char)(*(dst++))) >= _T( 'A' )) && (f <= _T( 'Z' )) )
           f -= (_T( 'A' ) - _T( 'a' ));

       if ( ((l = (unsigned char)(*(src++))) >= _T( 'A' )) && (l <= _T( 'Z' )) )
           l -= (_T( 'A' ) - _T( 'a' ));
   } while ( f && (f == l) );

   return(f - l);
}

//////////////////////////////////////////////////////////////////////

TCHAR	*sa_strlwr(TCHAR	*string)
{
   TCHAR	* cp;

   for (cp=string; *cp; ++cp)
   {
          if (_T( 'A' ) <= *cp && *cp <= _T( 'Z' ))
                  *cp += _T( 'a' ) - _T( 'A' );
   }

   return(string);
}

//////////////////////////////////////////////////////////////////////

TCHAR	*sa_strupr(TCHAR *string)
{
   TCHAR	* cp;

   for (cp=string; *cp; ++cp)
   {
      if (_T( 'a' ) <= *cp && *cp <= _T( 'z' ))
            *cp += (TCHAR)( _T( 'A' ) - _T( 'a' ) );
   }

   return(string);
}
