/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#include "stdafx.h"
#include "SAScrVariable.h"
#include "SAScrUtils.h"

////////////////////////////////////////////////////////////////

CSAScrVariable::CSAScrVariable()
{
   dwContext = 0;
}

////////////////////////////////////////////////////////////////

CSAScrVariable::CSAScrVariable(CSAString pName, CSAString pDesc, CSAString pVal, unsigned long dwCtxt)
{
   pVarName = pName;
   m_csDescription = pDesc;
   pValue = pVal;
   dwContext = dwCtxt;
}

////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void CSAScrVariable::Dump()
{
   SAScr::SAScrTrace(_T( "%s [\"%s\", %d] = \"%s\"\n" ), (const TCHAR*)pVarName, (const TCHAR *)m_csDescription, dwContext, (const TCHAR *)pValue);
}
#endif

////////////////////////////////////////////////////////////////

void CSAScrVariableMap::Add(const TCHAR	*pName, const TCHAR	*pValue, unsigned long dwContext)
{
   Add(pName, _T( "" ), pValue, dwContext);
}

////////////////////////////////////////////////////////////////

void CSAScrVariableMap::Add(const TCHAR	*pName, const TCHAR	*pDesc, const TCHAR	*pValue, unsigned long dwContext)
{
   CSAString csName(pName);
   csName.MakeUpper();

   CSAScrVariable v(pName, pDesc, pValue, dwContext);

   // clear duplicates
   iterator it = find(csName);
   if (it!=end())
   {
      erase(it);
   }
   pair<CSAScrVariableMap::iterator, bool> res = insert(CSAScrVariableMap::value_type(csName, v));
}

////////////////////////////////////////////////////////////////

void CSAScrVariableMap::Add(const CSAScrVariable &_v)
{
   CSAString csName(_v.pVarName);
   csName.MakeUpper();

   CSAScrVariable v(_v);

   pair<CSAScrVariableMap::iterator, bool> res = insert(CSAScrVariableMap::value_type(csName, v));
}

////////////////////////////////////////////////////////////////
bool operator < (const CSAScrVariable &a, const CSAScrVariable &b)
{
   return a.pVarName < b.pVarName;
}

////////////////////////////////////////////////////////////////

bool operator == (const CSAScrVariable &a, const CSAScrVariable &b)
{
   return (a.pVarName == b.pVarName);
}

////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void CSAScrVariableMap::Dump()
{
   iterator theIterator;
   for (theIterator=begin(); theIterator!=end(); theIterator++)
   {
      CSAScrVariable cur = (*theIterator).second;
#ifndef SASCR_SILENT
      cur.Dump();
#endif
   }
}
#endif