#include "Console.h"
#include <time.h>
#include "../MainWnd.h"


HWND WindowConsole = NULL; // extern
int iNext = 0;

const int iMaxEntries = 10000;

extern CMainWnd *pMainFrame;

WNDPROC WndprocConsoleBackup = NULL;
LRESULT CALLBACK WndprocConsole( HWND hwnd, UINT message, WPARAM wp, LPARAM lp );



bool bConsoleVisible;
WINDOWPLACEMENT WinPlaceConsole;

RECT rConsoleDefault = { 50, 50, 1000, 700 };

//////////////////////////////////////////////////////////////////////////////// 
/// 
//////////////////////////////////////////////////////////////////////////////// 
bool Console::Create()
{
	WindowConsole = CreateWindowEx(
		WS_EX_TOOLWINDOW |                             // DWORD dwExStyle
			WS_EX_CLIENTEDGE,                          //
		TEXT( "LISTBOX" ),                             // LPCTSTR lpClassName
		TEXT( "Console" ),                             // LPCTSTR lpWindowName
		WS_VSCROLL |                                   // DWORD dwStyle
			LBS_DISABLENOSCROLL |                      //
			LBS_EXTENDEDSEL |                          //
			LBS_HASSTRINGS |                           //
			LBS_NOTIFY |                               //
			LBS_NOINTEGRALHEIGHT |                     //
			WS_POPUP |                                 //
			WS_OVERLAPPEDWINDOW,                       //
		rConsoleDefault.left,                          // int x
		rConsoleDefault.top,                           // int y
		rConsoleDefault.right - rConsoleDefault.left,  // int nWidth
		rConsoleDefault.bottom - rConsoleDefault.top,  // int nHeight
		AfxGetApp()->m_pMainWnd->m_hWnd,                                    // HWND hWndParent
		NULL,                                          // HMENU hMenu
		AfxGetInstanceHandle(),                                   // HINSTANCE hInstance
		NULL                                           // LPVOID lpParam
	);
	
	if( !WindowConsole ) return false;

	SendMessage( WindowConsole, LB_INSERTSTRING, 0, ( LPARAM )TEXT( "" ) );
	
	bConsoleVisible = ( WinPlaceConsole.showCmd != SW_HIDE );
	SetWindowPlacement( WindowConsole, &WinPlaceConsole );
	
	// Exchange window procedure
	WndprocConsoleBackup = ( WNDPROC )GetWindowLong( WindowConsole, GWL_WNDPROC );
	if( WndprocConsoleBackup != NULL )
	{
		SetWindowLong( WindowConsole, GWL_WNDPROC, ( LONG )WndprocConsole );
	}

	
	return true;
}



//////////////////////////////////////////////////////////////////////////////// 
/// 
//////////////////////////////////////////////////////////////////////////////// 
bool Console::Destroy()
{
	if( !WindowConsole ) return false;
	DestroyWindow( WindowConsole );
	return true;
}



//////////////////////////////////////////////////////////////////////////////// 
/// 
//////////////////////////////////////////////////////////////////////////////// 
bool Console::Popup()
{
	if( !WindowConsole ) return false;	
	if( !IsWindowVisible( WindowConsole ) )
	{
		ShowWindow( WindowConsole, SW_SHOW );
	}
	
	SetActiveWindow( WindowConsole );
	
	return true;
}



//////////////////////////////////////////////////////////////////////////////// 
/// 
//////////////////////////////////////////////////////////////////////////////// 
bool Console::Append( TCHAR * szText )
{
	if( !WindowConsole || !pMainFrame->displayConsole ) return false;
	if( iNext > iMaxEntries - 1 )
	{
		SendMessage(
			WindowConsole,
			LB_DELETESTRING,
			0,
			0
		);
		iNext--;
	}
	
	const int uTextLen = ( int )_tcslen( szText );
	TCHAR * szBuffer = new TCHAR[ 11 + uTextLen + 1 ];
	time_t now_time_t = time( NULL );
	struct tm * now_tm = localtime( &now_time_t );
	_tcsftime( szBuffer, 12, TEXT( "%H:%M:%S   " ), now_tm );
	memcpy( szBuffer + 11, szText, uTextLen * sizeof( TCHAR ) );
	szBuffer[ 11 + uTextLen ] = TEXT( '\0' );
	
	if(IsWindow( WindowConsole )) {
		HFONT hFont = CreateFont(
				-11,                          // int nHeight
				0,                            // int nWidth
				0,                            // int nEscapement
				0,                            // int nOrientation
				FW_REGULAR,                   // int fnWeight
				FALSE,                        // DWORD fdwItalic
				FALSE,                        // DWORD fdwUnderline
				FALSE,                        // DWORD fdwStrikeOut
				DEFAULT_CHARSET,                 // DWORD fdwCharSet
				OUT_TT_PRECIS,                // DWORD fdwOutputPrecision
				CLIP_DEFAULT_PRECIS,          // DWORD fdwClipPrecision
				ANTIALIASED_QUALITY,          // DWORD fdwQuality
				FF_DONTCARE | DEFAULT_PITCH,  // DWORD fdwPitchAndFamily
				TEXT( "Verdana" )             // LPCTSTR lpszFace
			);
		::SendMessage( WindowConsole, WM_SETFONT, ( WPARAM )hFont, FALSE);
		::SendMessage( WindowConsole, LB_INSERTSTRING, iNext, ( LPARAM )szBuffer );
		::SendMessage( WindowConsole, LB_SETSEL, FALSE, -1 );
		::SendMessage( WindowConsole, LB_SETSEL, TRUE, iNext );
		::SendMessage( WindowConsole, LB_SETTOPINDEX, iNext, 0 );
	}
	iNext++;
	delete [] szBuffer;
	
	return true;
}



//////////////////////////////////////////////////////////////////////////////// 
/// 
//////////////////////////////////////////////////////////////////////////////// 
LRESULT CALLBACK WndprocConsole( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
	case WM_SYSCOMMAND:
		if( ( wp & 0xFFF0 ) == SC_CLOSE )
		{
			ShowWindow( hwnd, SW_HIDE );
			return 0;
		}
		break;
		
	case WM_DESTROY:
		break;

	case WM_SHOWWINDOW:
		bConsoleVisible = ( wp == TRUE );
		break;

	}
	return CallWindowProc( WndprocConsoleBackup, hwnd, message, wp, lp );
}
